package com.atguigu1.connection;

import org.junit.Test;

import java.io.InputStream;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.util.Properties;

/**
 * @Author: ZZP
 * @Description:
 *
 * @Date: Created in 20:07 2021/6/15
 * @Version: 1.0
 **/
public class ConnectionTest {
    // 方式一
    @Test
    public void testConnection1() throws Exception {
        // 获取 Driver 实现类对象
        Driver driver = new com.mysql.cj.jdbc.Driver();

        // jdbc:mysql:协议
        // localhost:ip地址
        // 3306:默认mysql的端口号
        // test:test数据库
        String url = "jdbc:mysql://localhost:3306/test?useUnicode=true&characterEncoding=UTF-8";

        // 将用户名和密码封装在Properties中
        Properties info = new Properties();
        info.setProperty("user", "root");
        info.setProperty("password", "719238");

        Connection conn = driver.connect(url, info);

        System.out.println(conn);
    }

    // 方式二：对方式一的迭代：在如下的程序中不出现第三方的api，使得程序具有更好的可移植性
    @Test
    public void testConnection2() throws Exception {
        // 1.获取 Driver 实现类对象，使用反射实现
        Class clazz = Class.forName("com.mysql.cj.jdbc.Driver");
        Driver driver = (Driver) clazz.newInstance();

        // 2.提供要连接的数据库
        String url = "jdbc:mysql://localhost:3306/test?useUnicode=true&characterEncoding=UTF-8";

        // 3.提供连接需要的用户名和密码
        Properties info = new Properties();
        info.setProperty("user", "root");
        info.setProperty("password", "719238");

        // 4.获取连接
        Connection conn = driver.connect(url, info);

        System.out.println(conn);
    }

    // 方式三：使用 DriverManager 替换 Driver
    @Test
    public void testConnection3() throws Exception {
        // 1.获取 Driver 实现类对象，使用反射实现
        Class clazz = Class.forName("com.mysql.jdbc.Driver");
        Driver driver = (Driver) clazz.newInstance();

        // 2.提供另外三个连接的基本信息
        String url = "jdbc:mysql://localhost:3306/test?useUnicode=true&characterEncoding=UTF-8";
        String user = "root";
        String password = "719238";

        // 注册驱动
        DriverManager.registerDriver(driver);

        // 3.获取链接
        Connection conn = DriverManager.getConnection(url, user, password);

        System.out.println(conn);
    }

    // 方式四：相较于方式三可以省略 注册驱动 这一步操作
    @Test
    public void testConnection4() throws Exception {
        // 1.提供另外三个连接的基本信息
        String url = "jdbc:mysql://localhost:3306/test?useUnicode=true&characterEncoding=UTF-8";
        String user = "root";
        String password = "719238";

        // 2.加载 Driver 实现类对象，使用反射实现
        Class.forName("com.mysql.jdbc.Driver");

        // 3.获取链接
        Connection conn = DriverManager.getConnection(url, user, password);

        System.out.println(conn);
    }

    // 方式五：将数据库连接需要的四个基本信息声明在配置文件中，通过读取配置文件，获取连接
    /*
    此种方式的好处：
    1. 实现了数据与代码的分离。
    2. 如果需要修改配置文件信息，可避免程序重新打包
     */
    @Test
    public void testConnection5() throws Exception {
        // 读取配置文件中的四个基本信息
        InputStream is = ConnectionTest.class.getClassLoader().getResourceAsStream("jdbc.properties");

        Properties pros = new Properties();
        pros.load(is);

        String user = pros.getProperty("user");
        String password = pros.getProperty("password");
        String url = pros.getProperty("url");
        String driverClass = pros.getProperty("driverClass");

        // 2.加载驱动
        Class.forName(driverClass);

        // 3.获取链接
        Connection conn = DriverManager.getConnection(url, user, password);

        System.out.println(conn);
    }
}
