package com.atguigu3.preparedstatement.curd;

import com.atguigu3.bean.Customer;
import com.atguigu3.util.JDBCUtils;

import java.lang.reflect.Field;
import java.sql.*;

/**
 * @Author: ZZP
 * @Description:
 * 针对于 Customers 表的查询操作
 *
 * @Date: Created in 12:25 2021/6/16
 * @Version: 1.0
 **/
public class CustomerForQuery {
    public static void main(String[] args) {
        testQuery1();
        System.out.println("----------------------------------------------------------");

        String sql = "select id, name, email, birth from customers where id = ?";
        Customer customer = queryForCustomer(sql, 1);
        System.out.println(customer);
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/16
     * @Description: 针对于 customers 表的通用查询操作
     */
    public static Customer queryForCustomer(String sql, Object... args) {
        Connection conn = null;
        PreparedStatement ps = null;
        ResultSet resultSet = null;
        try {
            // 1.获取数据库的连接
            conn = JDBCUtils.getConnection();

            // 2.预编译 sql 语句，返回 PreparedStatement 的实例
            ps = conn.prepareStatement(sql);

            // 3.填充占位符
            for (int i = 0; i < args.length; i++) {
                ps.setObject(i + 1, args[i]);
            }

            // 4.执行并返回结果集
            resultSet = ps.executeQuery();

            // 获取结果集的元数据：ResultSetMetaData
            ResultSetMetaData rsmd = resultSet.getMetaData();

            // 通过 ResultSetMetaData 获取结果集中的列数
            int columnCount = rsmd.getColumnCount();

            // 5.处理结果集
            if (resultSet.next()) {
                Customer customer = new Customer();

                // 处理结果集一行数据中的每一个列
                for (int i = 0; i < columnCount; i++) {
                    // 获取列值
                    Object columnValue = resultSet.getObject(i + 1);

                    // 获取每个列的列名
                    String columnName = rsmd.getColumnName(i + 1);

                    // 给 columnName 对象指定的某个属性，赋值为 columnValue : 通过反射
                    Field field = Customer.class.getDeclaredField(columnName);
                    field.setAccessible(true);
                    field.set(customer, columnValue);
                }
                return customer;
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // 关闭资源
            JDBCUtils.closeResource(conn, ps, resultSet);
        }
        return null;
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/16
     * @Description: 针对于 customers 表的查询操作
     */
    // 查询操作
    public static void testQuery1() {
        Connection conn = null;
        PreparedStatement ps = null;
        ResultSet resultSet = null;
        try {
            // 1.获取数据库的连接
            conn = JDBCUtils.getConnection();

            // 2.预编译 sql 语句，返回 PreparedStatement 的实例
            String sql = "select id, name, email, birth from customers where id = ?";
            ps = conn.prepareStatement(sql);

            // 3.填充占位符
            ps.setInt(1, 13);

            // 4.执行并返回结果集
            resultSet = ps.executeQuery();

            // 5.处理结果集
            if (resultSet.next()) { // 判断结果集的下一条是否有数据，如果有数据返回true，并且指针下移，如果没有数据返回false，并且指针不会下移
                // 获取当前这条数据的各个字段值
                int id = resultSet.getInt(1);
                String name = resultSet.getString(2);
                String email = resultSet.getString(3);
                Date birth = resultSet.getDate(4);

                // 方式一
                //Object[] data = new Object[]{id, name, email, birth};

                // 方式二：将数据封装为一个对象
                Customer customer = new Customer(id, name, email, birth);
                System.out.println(customer);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // 6.关闭资源
            JDBCUtils.closeResource(conn, ps, resultSet);
        }
    }
}
