package com.atguigu3.preparedstatement.curd;

import com.atguigu3.bean.Order;
import com.atguigu3.util.JDBCUtils;

import java.lang.reflect.Field;
import java.sql.*;

/**
 * @Author: ZZP
 * @Description:
 * 针对于 Order 表的通用的查询操作
 *
 * @Date: Created in 15:40 2021/6/16
 * @Version: 1.0
 **/
public class OrderForQuery {
    public static void main(String[] args) {
        testQuery();
        System.out.println("----------------------------------------------------------");

        String sql = "select order_id orderId, order_name orderName, order_date orderDate from `order` where order_id = ?";
        Order order = orderForQuery(sql, 1);
        System.out.println(order);
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/17
     * @Description: 针对于 order 表的查询操作
     */
    public static Order orderForQuery(String sql, Object... args) {
        /**
         * 针对于表的字段名与类的属性名不相同的情况：
         *      1.必须声明sql时，使用类的属性名来命名字段的别名
         *      2.使用 ResultSetMetaData 时，需要使用 getColumnLabel() 来替换 getColumnName() ，获取列的别名
         *
         *      说明：如果 sql 中没有给字段起别名，getColumnLabel() 获取的就是列名
         */
        Connection conn = null;
        PreparedStatement ps = null;
        ResultSet resultSet = null;
        try {
            conn = JDBCUtils.getConnection();

            // 2.预编译 sql 语句，返回 PreparedStatement 的实例
            ps = conn.prepareStatement(sql);

            // 3.填充占位符
            for (int i = 0; i < args.length; i++) {
                ps.setObject(i + 1, args[i]);
            }

            // 4.执行并返回结果集
            resultSet = ps.executeQuery();

            // 获取结果集的元数据：ResultSetMetaData
            ResultSetMetaData rsmd = resultSet.getMetaData();

            // 通过 ResultSetMetaData 获取结果集中的列数
            int columnCount = rsmd.getColumnCount();

            // 5.处理结果集
            if (resultSet.next()) {
                Order order = new Order();

                // 处理结果集一行数据中的每一个列
                for (int i = 0; i < columnCount; i++) {
                    // 获取列值
                    Object columnValue = resultSet.getObject(i + 1);

                    // 获取每个列的列名 getColumnName()
                    // 获取每个列的别名 getColumnLabel()
                    String columnLabel = rsmd.getColumnLabel(i + 1);

                    // 给 columnName 对象指定的某个属性，赋值为 columnValue : 通过反射
                    Field field = Order.class.getDeclaredField(columnLabel);
                    field.setAccessible(true);
                    field.set(order, columnValue);
                }
                return order;
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // 关闭资源
            JDBCUtils.closeResource(conn, ps, resultSet);
        }
        return null;
    }

    public static void testQuery() {
        Connection conn = null;
        PreparedStatement ps = null;
        ResultSet rs = null;
        try {
            conn = JDBCUtils.getConnection();

            String sql = "select order_id, order_name, order_date from `order` where order_id = ?";
            ps = conn.prepareStatement(sql);

            ps.setObject(1, 1);

            rs = ps.executeQuery();
            if (rs.next()) {
                int id = (int) rs.getObject(1);
                String name = (String) rs.getObject(2);
                Date date = (Date) rs.getObject(3);

                Order order = new Order(id, name, date);
                System.out.println(order);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.closeResource(conn, ps, rs);
        }
    }
}
