package com.atguigu3.preparedstatement.curd;

import com.atguigu3.bean.Customer;
import com.atguigu3.bean.Order;
import com.atguigu3.util.JDBCUtils;

import java.lang.reflect.Field;
import java.sql.*;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author: ZZP
 * @Description:
 * 使用 PreparedStatement 实现针对于不同表的通用的查询操作
 *
 * @Date: Created in 15:27 2021/6/17
 * @Version: 1.0
 **/
public class PreparedStatementQueryTest {
    public static void main(String[] args) {
        String sql = "select order_id orderId, order_name orderName, order_date orderDate from `order` where order_id = ?";
        Order order = getInstance(Order.class, sql, 1);
        System.out.println(order);

        System.out.println("--------------------------------------------------------------------------");
        String sql1 = "select id, name, email, birth from customers where id = ?";
        Customer customer = getInstance(Customer.class, sql1, 13);
        System.out.println(customer);

        System.out.println("--------------------------------------------------------------------------");
        String sql2 = "select id, name, email, birth from customers where id < ?";
        List<Customer> customerList = getForList(Customer.class, sql2, 13);
        customerList.forEach(System.out::println);
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/17
     * @Description: 针对于不同表的通用的查询操作，返回表中的多条记录
     */
    public static <T> List<T> getForList(Class<T> clazz, String sql, Object... args) {
        Connection conn = null;
        PreparedStatement ps = null;
        ResultSet resultSet = null;
        try {
            conn = JDBCUtils.getConnection();

            // 2.预编译 sql 语句，返回 PreparedStatement 的实例
            ps = conn.prepareStatement(sql);

            // 3.填充占位符
            for (int i = 0; i < args.length; i++) {
                ps.setObject(i + 1, args[i]);
            }

            // 4.执行并返回结果集
            resultSet = ps.executeQuery();

            // 获取结果集的元数据：ResultSetMetaData
            ResultSetMetaData rsmd = resultSet.getMetaData();

            // 通过 ResultSetMetaData 获取结果集中的列数
            int columnCount = rsmd.getColumnCount();

            // 创建集合对象
            ArrayList<T> list = new ArrayList<T>();

            // 5.处理结果集
            while (resultSet.next()) {
                T t = clazz.newInstance();

                // 处理结果集一行数据中的每一个列:给 t 对象指定的属性赋值
                for (int i = 0; i < columnCount; i++) {
                    // 获取列值
                    Object columnValue = resultSet.getObject(i + 1);

                    // 获取每个列的列名 getColumnName()
                    // 获取每个列的别名 getColumnLabel()
                    String columnLabel = rsmd.getColumnLabel(i + 1);

                    // 给 columnName 对象指定的某个属性，赋值为 columnValue : 通过反射
                    Field field = clazz.getDeclaredField(columnLabel);
                    field.setAccessible(true);
                    field.set(t, columnValue);
                }
                list.add(t);
            }
            return list;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // 关闭资源
            JDBCUtils.closeResource(conn, ps, resultSet);
        }
        return null;
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/17
     * @Description: 针对于不同表的通用的查询操作，返回表中的一条记录
     */
    public static <T> T getInstance(Class<T> clazz, String sql, Object... args) {
        Connection conn = null;
        PreparedStatement ps = null;
        ResultSet resultSet = null;
        try {
            conn = JDBCUtils.getConnection();

            // 2.预编译 sql 语句，返回 PreparedStatement 的实例
            ps = conn.prepareStatement(sql);

            // 3.填充占位符
            for (int i = 0; i < args.length; i++) {
                ps.setObject(i + 1, args[i]);
            }

            // 4.执行并返回结果集
            resultSet = ps.executeQuery();

            // 获取结果集的元数据：ResultSetMetaData
            ResultSetMetaData rsmd = resultSet.getMetaData();

            // 通过 ResultSetMetaData 获取结果集中的列数
            int columnCount = rsmd.getColumnCount();

            // 5.处理结果集
            if (resultSet.next()) {
                T t = clazz.newInstance();

                // 处理结果集一行数据中的每一个列
                for (int i = 0; i < columnCount; i++) {
                    // 获取列值
                    Object columnValue = resultSet.getObject(i + 1);

                    // 获取每个列的列名 getColumnName()
                    // 获取每个列的别名 getColumnLabel()
                    String columnLabel = rsmd.getColumnLabel(i + 1);

                    // 给 columnName 对象指定的某个属性，赋值为 columnValue : 通过反射
                    Field field = clazz.getDeclaredField(columnLabel);
                    field.setAccessible(true);
                    field.set(t, columnValue);
                }
                return t;
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // 关闭资源
            JDBCUtils.closeResource(conn, ps, resultSet);
        }
        return null;
    }
}
