package com.atguigu3.preparedstatement.curd;

import com.atguigu1.connection.ConnectionTest;
import org.junit.Test;

import java.io.IOException;
import java.io.InputStream;
import java.sql.*;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Properties;

/**
 * @Author: ZZP
 * @Description:
 * 使用 PreparedStatement 来替换 Statement，实现对数据表的增删改查操作
 *
 * 增删改
 * 查
 *
 * @Date: Created in 9:25 2021/6/16
 * @Version: 1.0
 **/
public class PreparedStatementUpdateTest {
    // 向 customer 表中添加一条记录
    public static void main(String[] args) {
        Connection conn = null;
        PreparedStatement ps = null;
        try {
            // 1.读取配置文件中的四个基本信息
            InputStream is = ClassLoader.getSystemClassLoader().getResourceAsStream("jdbc.properties");

            Properties pros = new Properties();
            pros.load(is);

            String user = pros.getProperty("user");
            String password = pros.getProperty("password");
            String url = pros.getProperty("url");
            String driverClass = pros.getProperty("driverClass");

            // 2.加载驱动
            Class.forName(driverClass);

            // 3.获取链接
            conn = DriverManager.getConnection(url, user, password);

            // 4.预编译 sql 语句，返回 PreparedStatement 的实例
            String sql = "insert into customers(name ,email,birth) value (?,?,?)";
            ps = conn.prepareStatement(sql);

            // 5.填充占位符
            ps.setString(1, "哪吒");
            ps.setString(2, "nezha@qq.com");
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            java.util.Date date = sdf.parse("1000-01-01");
            ps.setDate(3, new Date(date.getTime()));

            // 6.执行操作
            ps.execute();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        } catch (SQLException throwables) {
            throwables.printStackTrace();
        } catch (ParseException e) {
            e.printStackTrace();
        } finally {
            // 7.关闭资源
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException throwables) {
                    throwables.printStackTrace();
                }
            }

            if (ps != null) {
                try {
                    ps.close();
                } catch (SQLException throwables) {
                    throwables.printStackTrace();
                }
            }
        }
    }
}