package com.atguigu3.util;

import java.io.InputStream;
import java.sql.*;
import java.util.Properties;

/**
 * @Author: ZZP
 * @Description:
 * 操作数据库的工具类
 *
 * @Date: Created in 10:09 2021/6/16
 * @Version: 1.0
 **/
public class JDBCUtils {
    /**
     * @Author: ZZP
     * @Date:2021/6/16
     * @Description: 获取数据库的连接
     **/
    public static Connection getConnection() throws Exception {
        // 读取配置文件中的四个基本信息
        InputStream is = ClassLoader.getSystemClassLoader().getResourceAsStream("jdbc.properties");

        Properties pros = new Properties();
        pros.load(is);

        String user = pros.getProperty("user");
        String password = pros.getProperty("password");
        String url = pros.getProperty("url");
        String driverClass = pros.getProperty("driverClass");

        // 2.加载驱动
        Class.forName(driverClass);

        // 3.获取链接
        Connection conn = DriverManager.getConnection(url, user, password);

        return conn;
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/16
     * @Description: 关闭连接和 Statement 的操作
     */
    public static void closeResource(Connection conn, Statement ps){
        // 7.关闭资源
        if (conn != null) {
            try {
                conn.close();
            } catch (SQLException throwables) {
                throwables.printStackTrace();
            }
        }

        if (ps != null) {
            try {
                ps.close();
            } catch (SQLException throwables) {
                throwables.printStackTrace();
            }
        }
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/16
     * @Description: 关闭连接和 Statement 和 ResultSet 的操作
     */
    public static void closeResource(Connection conn, Statement ps, ResultSet rs){
        // 7.关闭资源
        if (conn != null) {
            try {
                conn.close();
            } catch (SQLException throwables) {
                throwables.printStackTrace();
            }
        }

        if (ps != null) {
            try {
                ps.close();
            } catch (SQLException throwables) {
                throwables.printStackTrace();
            }
        }

        if (rs != null) {
            try {
                rs.close();
            } catch (SQLException throwables) {
                throwables.printStackTrace();
            }
        }
    }
}
