package com.atguigu5.blob;

import com.atguigu3.bean.Customer;
import com.atguigu3.util.JDBCUtils;

import java.io.*;
import java.sql.*;

/**
 * @Author: ZZP
 * @Description:
 * 测试使用 PreparedStatement 操作 Blob 类型的数据
 *
 * @Date: Created in 21:43 2021/6/17
 * @Version: 1.0
 **/
public class BlobTest {
    public static void main(String[] args) {
        //int insert = testInsert();
        //if (insert > 0) {
        //    System.out.println("插入成功！");
        //} else {
        //    System.out.println("插入失败！");
        //}

        testQuery();
    }

    // 向数据表 customers 中插入 Blob 类型的字段
    public static int testInsert() {
        Connection conn = null;
        PreparedStatement ps = null;
        try {
            conn = JDBCUtils.getConnection();
            String sql = "insert into customers(name, email, birth, photo)values(?,?,?,?)";

            ps = conn.prepareStatement(sql);

            ps.setObject(1, "张浩宇");
            ps.setObject(2, "zhang@qq.com");
            ps.setObject(3, "1992-01-01");
            FileInputStream is = new FileInputStream(new File("123.jpg"));
            ps.setObject(4, is);

            return ps.executeUpdate();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.closeResource(conn, ps);
        }
        return 0;
    }

    // 查询数据表 customers 中的 Blob 类型的字段
    public static void testQuery() {
        Connection conn = null;
        PreparedStatement ps = null;
        ResultSet rs = null;
        InputStream is = null;
        FileOutputStream fos = null;
        try {
            conn = JDBCUtils.getConnection();

            String sql = "select id, name, email, birth, photo from customers where id = ?";
            ps = conn.prepareStatement(sql);

            ps.setObject(1, 22);

            rs = ps.executeQuery();

            is = null;
            fos = null;
            if (rs.next()) {
                // 方式一：
                //int id = rs.getInt(1);
                //String name = rs.getString(2);
                //String email = rs.getString(3);
                //Date birth = rs.getDate(4);
                //Object photo = rs.getObject(5);

                // 方式一：
                int id = rs.getInt("id");
                String name = rs.getString("name");
                String email = rs.getString("email");
                Date birth = rs.getDate("birth");

                Customer cust = new Customer(id, name, email, birth);
                System.out.println(cust);

                // 将 Blob 类型的字段下载下来，以文件的方式保存到本地
                Blob photo = rs.getBlob("photo");
                is = photo.getBinaryStream();
                fos = new FileOutputStream("zhuyin.jpg");
                byte[] buffer = new byte[1024];
                int len;
                while ((len = is.read(buffer)) != -1) {
                    fos.write(buffer, 0, len);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.closeResource(conn, ps, rs);

            try {
                if (is != null) {
                    is.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }

            try {
                if (fos != null) {
                    fos.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}
