package com.atguigu4.util;

import com.alibaba.druid.pool.DruidDataSourceFactory;
import com.mchange.v2.c3p0.ComboPooledDataSource;
import org.apache.commons.dbcp2.BasicDataSource;
import org.apache.commons.dbcp2.BasicDataSourceFactory;
import org.apache.commons.dbutils.DbUtils;

import javax.sql.DataSource;
import java.io.InputStream;
import java.sql.*;
import java.util.Properties;

/**
 * @Author: ZZP
 * @Date: Created in 16:38 2021/6/19
 * @Version: 1.0
 * @Description:
 **/
public class JDBCUtils {
    /**
     * @Author: ZZP
     * @Date:2021/6/19
     * @Version: 1.0
     * @Description: 使用 Druid 数据库连接池技术
     */
    private static DataSource source1;

    static {
        try {
            Properties pros = new Properties();

            InputStream is = ClassLoader.getSystemClassLoader().getResourceAsStream("druid.properties");

            pros.load(is);

            source1 = DruidDataSourceFactory.createDataSource(pros);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static Connection getConnection3() throws SQLException {
        Connection conn = source1.getConnection();
        return conn;
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/19
     * @Version: 1.0
     * @Description: 使用 C3P0 的数据库连接池技术
     */
    // C3P0 数据库连接池只需提供一个即可
    private static ComboPooledDataSource cpds = new ComboPooledDataSource("helloC3P0");

    public static Connection getConnection1() throws SQLException {
        Connection conn = cpds.getConnection();

        return conn;
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/19
     * @Version: 1.0
     * @Description: 使用 DBCP 的数据库连接池技术
     */
    // 创建一个 DBCP 数据库连接池
    private static BasicDataSource source;

    static {
        try {
            Properties pros = new Properties();

            InputStream is = ClassLoader.getSystemClassLoader().getResourceAsStream("dbcp.properties");

            pros.load(is);
            source = BasicDataSourceFactory.createDataSource(pros);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static Connection getConnection2() throws Exception {
        Connection conn = source.getConnection();

        return conn;
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/16
     * @Description: 获取数据库的连接
     **/
    public static Connection getConnection() throws Exception {
        // 读取配置文件中的四个基本信息
        InputStream is = ClassLoader.getSystemClassLoader().getResourceAsStream("jdbc.properties");

        Properties pros = new Properties();
        pros.load(is);

        String user = pros.getProperty("user");
        String password = pros.getProperty("password");
        String url = pros.getProperty("url");
        String driverClass = pros.getProperty("driverClass");

        // 2.加载驱动
        Class.forName(driverClass);

        // 3.获取链接
        Connection conn = DriverManager.getConnection(url, user, password);

        return conn;
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/16
     * @Description: 关闭连接和 Statement 的操作
     */
    public static void closeResource(Connection conn, Statement ps) {
        // 7.关闭资源
        if (conn != null) {
            try {
                conn.close();
            } catch (SQLException throwables) {
                throwables.printStackTrace();
            }
        }

        if (ps != null) {
            try {
                ps.close();
            } catch (SQLException throwables) {
                throwables.printStackTrace();
            }
        }
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/16
     * @Description: 关闭连接和 Statement 和 ResultSet 的操作
     */
    public static void closeResource(Connection conn, Statement ps, ResultSet rs) {
        // 7.关闭资源
        if (conn != null) {
            try {
                conn.close();
            } catch (SQLException throwables) {
                throwables.printStackTrace();
            }
        }

        if (ps != null) {
            try {
                ps.close();
            } catch (SQLException throwables) {
                throwables.printStackTrace();
            }
        }

        if (rs != null) {
            try {
                rs.close();
            } catch (SQLException throwables) {
                throwables.printStackTrace();
            }
        }
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/19
     * @Version: 1.0
     * @Description: 使用 dbutils。jar 中提供的 DbUtils 工具类，实现资源的关闭
     */
    public static void closeResource1(Connection conn, Statement ps, ResultSet rs) {
        //try {
        //    DbUtils.close(conn);
        //} catch (SQLException throwables) {
        //    throwables.printStackTrace();
        //}
        //
        //try {
        //    DbUtils.close(ps);
        //} catch (SQLException throwables) {
        //    throwables.printStackTrace();
        //}
        //
        //try {
        //    DbUtils.close(rs);
        //} catch (SQLException throwables) {
        //    throwables.printStackTrace();
        //}

        DbUtils.closeQuietly(conn, ps, rs);
    }
}
